"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var _ethers, _safeContract;
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../utils");
const constants_1 = require("../utils/constants");
class ModuleManager {
    constructor(ethers, safeContract) {
        _ethers.set(this, void 0);
        _safeContract.set(this, void 0);
        __classPrivateFieldSet(this, _ethers, ethers);
        __classPrivateFieldSet(this, _safeContract, safeContract);
    }
    validateModuleAddress(moduleAddress) {
        const isValidAddress = __classPrivateFieldGet(this, _ethers).utils.isAddress(moduleAddress);
        if (!isValidAddress || utils_1.isRestrictedAddress(moduleAddress)) {
            throw new Error('Invalid module address provided');
        }
    }
    validateModuleIsNotEnabled(moduleAddress, modules) {
        const moduleIndex = modules.findIndex((module) => utils_1.sameString(module, moduleAddress));
        const isEnabled = moduleIndex >= 0;
        if (isEnabled) {
            throw new Error('Module provided is already enabled');
        }
    }
    validateModuleIsEnabled(moduleAddress, modules) {
        const moduleIndex = modules.findIndex((module) => utils_1.sameString(module, moduleAddress));
        const isEnabled = moduleIndex >= 0;
        if (!isEnabled) {
            throw new Error('Module provided is not enabled already');
        }
        return moduleIndex;
    }
    async getModules() {
        return __classPrivateFieldGet(this, _safeContract).getModules();
    }
    async isModuleEnabled(moduleAddress) {
        return __classPrivateFieldGet(this, _safeContract).isModuleEnabled(moduleAddress);
    }
    async encodeEnableModuleData(moduleAddress) {
        this.validateModuleAddress(moduleAddress);
        const modules = await this.getModules();
        this.validateModuleIsNotEnabled(moduleAddress, modules);
        return __classPrivateFieldGet(this, _safeContract).interface.encodeFunctionData('enableModule', [moduleAddress]);
    }
    async encodeDisableModuleData(moduleAddress) {
        this.validateModuleAddress(moduleAddress);
        const modules = await this.getModules();
        const moduleIndex = this.validateModuleIsEnabled(moduleAddress, modules);
        const prevModuleAddress = moduleIndex === 0 ? constants_1.SENTINEL_ADDRESS : modules[moduleIndex - 1];
        return __classPrivateFieldGet(this, _safeContract).interface.encodeFunctionData('disableModule', [
            prevModuleAddress,
            moduleAddress
        ]);
    }
}
_ethers = new WeakMap(), _safeContract = new WeakMap();
exports.default = ModuleManager;
//# sourceMappingURL=moduleManager.js.map